@@ -21,11 +21,15 @@ import org.gcube.common.storagehub.client.proxies.ItemManagerClient;
 import org.gcube.common.storagehub.client.proxies.WorkspaceManagerClient;
 import org.gcube.common.storagehub.model.Metadata;
 import org.gcube.common.storagehub.model.acls.ACL;
+import org.gcube.common.storagehub.model.exceptions.StorageHubException;
 import org.gcube.common.storagehub.model.items.AbstractFileItem;
+import org.gcube.common.storagehub.model.items.ExternalLink;
 import org.gcube.common.storagehub.model.items.FolderItem;
+import org.gcube.common.storagehub.model.items.GCubeItem;
 import org.gcube.common.storagehub.model.items.Item;
 import org.gcube.common.storagehub.model.items.SharedFolder;
 import org.gcube.common.storagehub.model.items.VreFolder;
+import org.gcube.common.storagehub.model.items.nodes.Accounting;
 import org.gcube.common.storagehub.model.items.nodes.ImageContent;
 import org.gcube.common.storagehub.model.service.Version;
 import org.gcube.common.storagehubwrapper.server.converter.ObjectMapper;
@@ -75,7 +79,7 @@ public class StorageHubClientService {
 	private void setContextProviders(String scope, String authorizationToken){
 		ScopeProvider.instance.set(scope);
 		SecurityTokenProvider.instance.set(authorizationToken);
-		logger.debug("Saved the contexts [scope: "+scope+", token: "+authorizationToken+"]");
+		logger.debug("Saved the contexts [scope: "+scope+", token: "+authorizationToken.substring(0,15)+"-MASKED]");
 	}
 
 	/**
@@ -110,7 +114,7 @@ public class StorageHubClientService {
 	 * @param withAccounting the with accounting
 	 * @param withMapProperties the with map properties
 	 * @return the children
-	 * @throws Exception 
+	 * @throws Exception the exception
 	 */
 	public List<? extends Item> getChildren(String id, boolean withAccounting, boolean withMapProperties) throws Exception{
 		setContextProviders(scope, authorizationToken);
@@ -125,6 +129,7 @@ public class StorageHubClientService {
 			theResolver = theResolver.withMetadata();
 
 		return theResolver.getItems();
+		
 	}
 
 
@@ -136,7 +141,7 @@ public class StorageHubClientService {
 	 * @param withAccounting the with accounting
 	 * @param withMapProperties the with map properties
 	 * @return the filtered children
-	 * @throws Exception 
+	 * @throws Exception the exception
 	 */
 	public List<? extends Item> getFilteredChildren(String id, Class<? extends Item> aType, boolean withAccounting, boolean withMapProperties) throws Exception{
 		setContextProviders(scope, authorizationToken);
@@ -149,7 +154,7 @@ public class StorageHubClientService {
 
 		if(withMapProperties)
 			theResolver = theResolver.withMetadata();
-
+		
 		return theResolver.getItems();
 	}
 
@@ -167,15 +172,16 @@ public class StorageHubClientService {
 		setContextProviders(scope, authorizationToken);
 		ItemContainer<Item> itemCont;
 
-		//MUST BE CHANGED. MOVE IT TO BUILDER?
-		if(withMetadata)
+		//I'M GOING TO REMAIN THESE OPTIONS FOR POSSIBLE FUTURE SUPPORTING ON SHUB
+		if(withMetadata) {
 			itemCont = shcClient.open(itemId).asItem(); //TODO
-		else if(withAccounting){
+		}else if(withAccounting){
 			itemCont = shcClient.open(itemId).asItem(); //TODO
 		}
-		else
+		else {
 			itemCont = shcClient.open(itemId).asItem();
-
+		}
+		
 		return itemCont.get();
 	}
 
@@ -245,7 +251,7 @@ public class StorageHubClientService {
 	 *
 	 * @param itemId the item id
 	 * @return the parents
-	 * @throws Exception 
+	 * @throws Exception the exception
 	 */
 	public List<? extends Item> getParents(String itemId) throws Exception {
 		setContextProviders(scope, authorizationToken);
@@ -299,7 +305,21 @@ public class StorageHubClientService {
 		return shcClient.open(itemId).asItem().get().isShared();
 
 	}
+	
+	
+	/**
+	 * Can write.
+	 *
+	 * @param folderContainerId the folder container id
+	 * @return true, if true. Otherwise false.
+	 * @throws Exception the exception
+	 */
+	public boolean canWrite(String folderContainerId) throws Exception {
+		setContextProviders(scope, authorizationToken);
+		return shcClient.open(folderContainerId).asFolder().canWrite();
+	}
 
+	
 	/**
 	 * Gets the root shared folder.
 	 *
@@ -407,7 +427,7 @@ public class StorageHubClientService {
 	 *
 	 * @param itemId the item id
 	 * @return the item children count
-	 * @throws Exception 
+	 * @throws Exception the exception
 	 */
 	public int getItemChildrenCount(String itemId) throws Exception {
 
@@ -547,7 +567,7 @@ public class StorageHubClientService {
 	 */
 	public void deleteItemById(String itemId) throws Exception{
 
-		Validate.notNull(itemId, "Bad request of deleteItemById, the itemId is null");
+		Validate.notNull(itemId, "Bad request to deleteItemById, the itemId is null");
 		setContextProviders(scope, authorizationToken);
 		shcClient.open(itemId).asItem().delete();
 
@@ -592,7 +612,7 @@ public class StorageHubClientService {
 	 */
 	public Item restoreThrashItem(String itemId) throws Exception{
 
-		Validate.notNull(itemId, "Bad request of restoreThrashItem, the itemId is null");
+		Validate.notNull(itemId, "Bad request to restoreThrashItem, the itemId is null");
 		setContextProviders(scope, authorizationToken);
 		GenericItemContainer container = shcClient.restoreThrashItem(itemId);
 
@@ -618,8 +638,8 @@ public class StorageHubClientService {
 	 */
 	public Item moveItem(String itemId, FolderContainer destFolderContainer) throws Exception{
 
-		Validate.notNull(itemId, "Bad request of moveItem, the itemId is null");
-		Validate.notNull(destFolderContainer, "Bad request of moveItem, the itemId is null");
+		Validate.notNull(itemId, "Bad request to moveItem, the itemId is null");
+		Validate.notNull(destFolderContainer, "Bad request to moveItem, the itemId is null");
 		setContextProviders(scope, authorizationToken);
 		shcClient.open(itemId).asItem().move(destFolderContainer);
 		return shcClient.open(itemId).asItem().get();
@@ -638,8 +658,8 @@ public class StorageHubClientService {
 	 */
 	public AbstractFileItem copyFileItem(String fileItemId, FolderContainer destFolderContainer, String newFileName) throws Exception{
 
-		Validate.notNull(fileItemId, "Bad request of copyFileItem, the fileItemId is null");
-		Validate.notNull(destFolderContainer, "Bad request of copyFileItem, the destFolderContainer is null");
+		Validate.notNull(fileItemId, "Bad request to copyFileItem, the fileItemId is null");
+		Validate.notNull(destFolderContainer, "Bad request to copyFileItem, the destFolderContainer is null");
 		setContextProviders(scope, authorizationToken);
 		FileContainer copyingItem = shcClient.open(fileItemId).asFile();
 		String newName = newFileName!=null && !newFileName.isEmpty()?newFileName:"Copy of "+copyingItem.get().getName();
@@ -658,7 +678,7 @@ public class StorageHubClientService {
 	 */
 	public Item renameItem(String itemId, String newName) throws Exception{
 
-		Validate.notNull(itemId, "Bad request of renameItem, the itemId is null");
+		Validate.notNull(itemId, "Bad request to renameItem, the itemId is null");
 		setContextProviders(scope, authorizationToken);
 		shcClient.open(itemId).asItem().rename(newName);
 		return shcClient.open(itemId).asItem().get();
@@ -675,7 +695,7 @@ public class StorageHubClientService {
 	 */
 	public URL getPublicLinkForFile(String fileItemId) throws Exception{
 
-		Validate.notNull(fileItemId, "Bad request of getPublicLinkForFile, the fileItemId is null");
+		Validate.notNull(fileItemId, "Bad request to getPublicLinkForFile, the fileItemId is null");
 		setContextProviders(scope, authorizationToken);
 		return shcClient.open(fileItemId).asFile().getPublicLink();
 
@@ -686,14 +706,15 @@ public class StorageHubClientService {
 	 * Gets the public link for file version.
 	 *
 	 * @param fileItemId the file item id
+	 * @param version the version
 	 * @return the public link for file version
 	 * @throws Exception the exception
 	 */
 	public URL getPublicLinkForFileVersion(String fileItemId, String version) throws Exception{
 
-		Validate.notNull(fileItemId, "Bad request of getPublicLinkForFile, the fileItemId is null");
-		Validate.notNull(version, "Bad request of getPublicLinkForFileVersion, the version is null");
-		Validate.notEmpty(version, "Bad request of getPublicLinkForFileVersion, the version is empty");
+		Validate.notNull(fileItemId, "Bad request to getPublicLinkForFile, the fileItemId is null");
+		Validate.notNull(version, "Bad request to getPublicLinkForFileVersion, the version is null");
+		Validate.notEmpty(version, "Bad request to getPublicLinkForFileVersion, the version is empty");
 		setContextProviders(scope, authorizationToken);
 		return shcClient.open(fileItemId).asFile().getPublicLink(version);
 
@@ -709,7 +730,7 @@ public class StorageHubClientService {
 	 */
 	public List<Version> getListVersions(String fileItemId) throws Exception{
 
-		Validate.notNull(fileItemId, "Bad request of getListVersions, the fileItemId is null");
+		Validate.notNull(fileItemId, "Bad request to getListVersions, the fileItemId is null");
 
 		return shcClient.open(fileItemId).asFile().getVersions();
 
@@ -724,18 +745,143 @@ public class StorageHubClientService {
 	 */
 	public ImageContent getImageContent(String itemId) throws Exception{
 
-		Validate.notNull(itemId, "Bad request of getThumbnailData, the itemId is null");
+		Validate.notNull(itemId, "Bad request to getImageContent, the itemId is null");
 		setContextProviders(scope, authorizationToken);
 		ItemContainer<Item> itemCont = shcClient.open(itemId).asItem();
 		Item item = itemCont.get();
 		if(item instanceof org.gcube.common.storagehub.model.items.ImageFile){
-
     		org.gcube.common.storagehub.model.items.ImageFile imgFI = (org.gcube.common.storagehub.model.items.ImageFile) item; //??
     		return imgFI.getContent();
 		}else
 			throw new Exception("Thumbnail Data is not available for type: "+item.getClass().getSimpleName());
 	}
 
+	
+	/**
+	 * Gets the total items.
+	 *
+	 * @return the total items
+	 * @throws Exception the exception
+	 */
+	public long getTotalItems() throws Exception{
+
+		setContextProviders(scope, authorizationToken);
+		return shcClient.getTotalItemCount();
+	}
+	
+	
+	/**
+	 * Gets the disk usage.
+	 *
+	 * @return the disk usage
+	 * @throws Exception the exception
+	 */
+	public long getDiskUsage() throws Exception{
+		setContextProviders(scope, authorizationToken);
+		return shcClient.getTotalVolume();
+	}
+	
+	/**
+	 * Gets the accounting.
+	 *
+	 * @param itemId the item id
+	 * @return the accounting
+	 * @throws Exception the exception
+	 */
+	public Accounting getAccounting(String itemId) throws Exception{
+
+		Validate.notNull(itemId, "Bad request to getAccounting, the itemId is null");
+		setContextProviders(scope, authorizationToken);
+		ItemContainer<Item> itemCont = shcClient.open(itemId).asItem();
+		Item item = itemCont.get();
+		return item.getAccounting();
+	}
+	
+	
+	/**
+	 * Gets the gcube item properties.
+	 *
+	 * @param gcubeItemId the gcube item id
+	 * @return the gcube item properties
+	 * @throws Exception the exception
+	 */
+	public Metadata getGcubeItemProperties(String gcubeItemId) throws Exception {
+		Validate.notNull(gcubeItemId, "Bad request to loadGcubeItemProperties, the itemId is null");
+		setContextProviders(scope, authorizationToken);
+		ItemContainer<Item> itemCont = shcClient.open(gcubeItemId).asItem();
+		Item item = itemCont.get();
+		
+		if(item instanceof GCubeItem) {
+			GCubeItem theGcubeItem = (GCubeItem) item; 
+			return theGcubeItem.getMetadata();
+		}else
+			throw new Exception("The item with id "+gcubeItemId+" is not an istance of "+GCubeItem.class.getSimpleName());
+	}
+	
+
+
+	/**
+	 * Sets the metadata.
+	 *
+	 * @param itemId the item id
+	 * @param mapProperties the map properties
+	 * @throws Exception the exception
+	 */
+	public void setMetadata(String itemId, Map<String, Object> mapProperties) throws Exception {
+		Validate.notNull(itemId, "Bad request to setMetadata the itemId is null");
+		setContextProviders(scope, authorizationToken);
+		//ItemContainer<Item> itemCont = shcClient.open(itemId).asItem();
+		//itemCont.get().setMetadata(new Metadata(mapProperties));
+		//item.setMetadata(new Metadata(mapProperties));
+		shcClient.open(itemId).asItem().setMetadata(new Metadata(mapProperties));
+	}
+
+	
+
+	/**
+	 * Adds the URL.
+	 *
+	 * @param folderId the folder id
+	 * @param URL the url
+	 * @param name the name
+	 * @param description the description
+	 * @return the external link
+	 * @throws StorageHubException the storage hub exception
+	 */
+	public ExternalLink addURL(String folderId, URL URL, String name, String description) throws StorageHubException {
+		Validate.notNull(folderId, "Bad request to createURL the folderId is null");
+		setContextProviders(scope, authorizationToken);
+		FolderContainer folder = shcClient.open(folderId).asFolder();
+		
+		return folder.addUrl(URL, name, description).get();
+	}
+	
+	
+	/**
+	 * Sets the folder as public.
+	 *
+	 * @param folderId the folder id
+	 * @param setPublic the set public
+	 * @return boolean value of isPublicItem()
+	 * @throws Exception the exception
+	 */
+	public boolean setFolderAsPublic(String folderId, boolean setPublic) throws Exception {
+		Validate.notNull(folderId, "Bad request to setFolderAsPublic the folderId is null");
+		setContextProviders(scope, authorizationToken);
+		FolderContainer folderCont = shcClient.open(folderId).asFolder();
+		
+		if(setPublic) {
+			folderCont.publish();
+			logger.debug("Published the folder id: "+folderId);
+		}else {
+			folderCont.unpublish();
+			logger.debug("Unpublished the folder id: "+folderId);
+		}
+		
+		return  shcClient.open(folderId).asFolder().get().isPublicItem();
+	}
+
+	
 
 	/* (non-Javadoc)
 	 * @see java.lang.Object#toString()
@@ -755,5 +901,4 @@ public class StorageHubClientService {
 	}
 
 
-
 }
