@@ -4,7 +4,6 @@
 package org.gcube.common.storagehubwrapper.server;
 
 import java.io.ByteArrayInputStream;
-import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
 import java.util.ArrayList;
@@ -16,12 +15,14 @@ import java.util.Map;
 import org.apache.commons.lang.Validate;
 import org.gcube.common.storagehub.client.StreamDescriptor;
 import org.gcube.common.storagehub.client.dsl.FolderContainer;
+import org.gcube.common.storagehub.model.Metadata;
 import org.gcube.common.storagehub.model.items.AbstractFileItem;
+import org.gcube.common.storagehub.model.items.ExternalLink;
 import org.gcube.common.storagehub.model.items.FolderItem;
 import org.gcube.common.storagehub.model.items.Item;
 import org.gcube.common.storagehub.model.items.nodes.ImageContent;
+import org.gcube.common.storagehub.model.items.nodes.accounting.AccountEntry;
 import org.gcube.common.storagehub.model.service.Version;
-import org.gcube.common.storagehub.model.types.GenericItemType;
 import org.gcube.common.storagehubwrapper.server.converter.HLMapper;
 import org.gcube.common.storagehubwrapper.server.tohl.Workspace;
 import org.gcube.common.storagehubwrapper.shared.ACLType;
@@ -35,10 +36,12 @@ import org.gcube.common.storagehubwrapper.shared.tohl.exceptions.ItemNotFoundExc
 import org.gcube.common.storagehubwrapper.shared.tohl.exceptions.WorkspaceFolderNotFoundException;
 import org.gcube.common.storagehubwrapper.shared.tohl.exceptions.WrongDestinationException;
 import org.gcube.common.storagehubwrapper.shared.tohl.exceptions.WrongItemTypeException;
+import org.gcube.common.storagehubwrapper.shared.tohl.impl.GcubeItem;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceFileVersion;
 import org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceFolder;
 import org.gcube.common.storagehubwrapper.shared.tohl.items.ItemStreamDescriptor;
-import org.gcube.common.storagehubwrapper.shared.tohl.items.URLFileItem;
+import org.gcube.common.storagehubwrapper.shared.tohl.items.PropertyMap;
+import org.gcube.common.storagehubwrapper.shared.tohl.items.URLItem;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -52,11 +55,13 @@ import org.slf4j.LoggerFactory;
 public final class WorkspaceStorageHubClientService implements Workspace{
 
 	//public static final String ACCOUNTING_HL_NODE_NAME = "hl:accounting";
-	private static Logger logger = LoggerFactory.getLogger(WorkspaceStorageHubClientService.class);
+	private static Logger LOGGER = LoggerFactory.getLogger(WorkspaceStorageHubClientService.class);
 	private StorageHubClientService storageHubClientService;
 	private boolean withAccounting;
 	private boolean withFileDetails;
 	private boolean withMapProperties;
+	
+	private WorkspaceFolder workspaceRoot = null; 
 
 	/**
 	 * Gets the storage hub client service.
@@ -227,25 +232,14 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	}
 
 
-	/* (non-Javadoc)
-	 * @see org.gcube.common.storagehubwrapper.server.tohl.Workspace#markAsPublic(java.lang.String, boolean)
-	 */
-	@Override
-	public Boolean markAsPublic(String itemId, boolean isPublic) {
-
-		//TODO
-		return false;
-	}
-
-
-
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getOwner()
 	 */
 	@Override
 	public String getOwner()
 		throws InternalErrorException, Exception {
-
+		
+		LOGGER.debug("called get Owner");
 		WorkspaceFolder root = getRoot();
 		return root.getOwner();
 	}
@@ -255,26 +249,53 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	 */
 	@Override
 	public WorkspaceFolder getRoot() throws InternalErrorException, Exception{
-		logger.debug("Getting root");
+		
+		LOGGER.debug("called get Root");
 		FolderItem root;
 		try {
 			root = storageHubClientService.getRoot();
 		}
 		catch (Exception e) {
-			logger.error("Error on getting root: ", e);
+			LOGGER.error("Error on getting root: ", e);
 			throw new InternalErrorException("Sorry an error occurred when getting the workspace root. Refresh and try again");
 		}
 		WorkspaceFolder workspaceFolder = (WorkspaceFolder) HLMapper.toWorkspaceItem(root, withAccounting, withFileDetails, withMapProperties);
 		workspaceFolder.setRoot(true);
+		workspaceRoot = workspaceFolder;
 		return workspaceFolder;
 	}
 
 
+	/**
+	 * We need to perform this method to set the root attribute.
+	 * No other way to get this information by SHUB
+	 *
+	 * @param item the item
+	 * @return the workspace item
+	 * @throws InternalErrorException the internal error exception
+	 * @throws Exception the exception
+	 */
+	private WorkspaceItem setIsRoot(org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceItem item) throws InternalErrorException, Exception {
+		
+		LOGGER.debug("called set Is Root");
+		Validate.notNull(item,"The input item is null");
+		if(workspaceRoot==null) {
+			LOGGER.debug("Workspace root instance is null into "+WorkspaceStorageHubClientService.class.getSimpleName()+", instancing it as first time");
+			getRoot();
+		}
+		boolean isRoot = workspaceRoot.getId().compareTo(item.getId())==0;
+		item.setRoot(isRoot);
+		LOGGER.info("Is the item '"+item.getName()+"' with id '"+item.getId()+"' the root? " +item.isRoot());
+		return item;
+	}
+
+
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.tohl.Workspace#getChildren(java.lang.String)
 	 */
 	public List<? extends WorkspaceItem> getChildren(String id) throws Exception{
-
+		
+		LOGGER.debug("called get Children");
 		Validate.notNull(id,"The input id is null");
 		List<? extends Item> children = storageHubClientService.getChildren(id, withAccounting, withMapProperties);
 		List<WorkspaceItem> toChildren = new ArrayList<WorkspaceItem>(children.size());
@@ -291,7 +312,8 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	 * @see org.gcube.portal.storagehubwrapper.shared.tohl.Workspace#getChildren(java.lang.String)
 	 */
 	public List<? extends WorkspaceItem> getFilteredChildren(String id, Class<? extends Item> aType) throws Exception{
-
+		
+		LOGGER.debug("called get Filtered Children");
 		Validate.notNull(id,"The input id is null");
 		List<? extends Item> children = storageHubClientService.getFilteredChildren(id, aType, withAccounting, withMapProperties);
 		List<WorkspaceItem> toChildren = new ArrayList<WorkspaceItem>(children.size());
@@ -310,13 +332,15 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	@Override
 	public List<? extends WorkspaceItem> getParentsById(String id)
 		throws InternalErrorException, Exception {
-
+		
+		LOGGER.debug("called get Parents by Id");
 		Validate.notNull(id,"The input id is null");
 		List<? extends Item> parents = storageHubClientService.getParents(id);
 		List<WorkspaceItem> toParents = new ArrayList<WorkspaceItem>(parents.size());
 
 		for (Item item : parents) {
 			WorkspaceItem child = HLMapper.toWorkspaceItem(item, withAccounting, withFileDetails, withMapProperties);
+			setIsRoot((org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceItem) child);
 			toParents.add(child);
 		}
 
@@ -332,7 +356,8 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		throws InternalErrorException, InsufficientPrivilegesException,
 		ItemAlreadyExistException, WrongDestinationException,
 		ItemNotFoundException, WorkspaceFolderNotFoundException {
-
+		
+		LOGGER.debug("called create Folder");
 		Validate.notNull(destinationFolderId,"The destinationFolderId id is null");
 		Validate.notNull(name,"The folderName is null");
 		Item item;
@@ -341,7 +366,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			return (WorkspaceFolder) HLMapper.toWorkspaceItem(item, withAccounting, withFileDetails, withMapProperties);
 		}
 		catch (Exception e) {
-			logger.error("Error on creating the folder: ",e);
+			LOGGER.error("Error on creating the folder: ",e);
 			throw new InternalErrorException(e.getMessage());
 		}
 
@@ -365,17 +390,20 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		String itemId, boolean withAccounting, boolean withFileDetails,
 		boolean withMapProperties)
 		throws ItemNotFoundException, InternalErrorException, Exception {
-
+		
+		LOGGER.debug("called get Item");
 		Validate.notNull(itemId,"The input itemId is null");
 		Item item;
 		try {
 			item = storageHubClientService.getItem(itemId, withAccounting, withMapProperties);
 		}
 		catch (Exception e) {
-			logger.error("Error during get item with id: "+itemId,e);
+			LOGGER.error("Error during get item with id: "+itemId,e);
 			throw new InternalErrorException(e.getMessage());
 		}
-		return HLMapper.toWorkspaceItem(item, withAccounting, withFileDetails, withMapProperties);
+		WorkspaceItem workspaceItem = HLMapper.toWorkspaceItem(item, withAccounting, withFileDetails, withMapProperties);
+		setIsRoot((org.gcube.common.storagehubwrapper.shared.tohl.impl.WorkspaceItem) workspaceItem);
+		return workspaceItem;
 	}
 
 
@@ -401,20 +429,17 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	@Override
 	public WorkspaceItem uploadFile(
 		String folderId, InputStream is, String fileName, String fileDescription)
-		throws InsufficientPrivilegesException,
-		WorkspaceFolderNotFoundException, InternalErrorException,
-		ItemAlreadyExistException, WrongDestinationException {
-
+		throws Exception {
+		
+		LOGGER.debug("called upload File");
 		WorkspaceItem wsItem = null;
 		try {
 			Item item = storageHubClientService.uploadFile(folderId, is, fileName, fileDescription);
 			wsItem = HLMapper.toWorkspaceItem(item);
 		}
 		catch (Exception e) {
-			logger.error("Error during uploading the file: "+fileName+" in the folderId: "+folderId, e);
-			String error = e.getMessage()!=null?e.getMessage():"";
-			throw new InternalErrorException("Error during uploading the file: "+fileName+". "+error);
-
+			LOGGER.error("Error uploading the file: "+fileName+" in the folderId: "+folderId, e);
+			throw e;
 		}
 
 		return wsItem;
@@ -430,10 +455,11 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	public List<String> getSharedFolderMembers(String folderId) throws Exception {
 
 		try{
+			LOGGER.debug("called Shared Folder Members");
 			Validate.notNull(folderId,"The input folderid is null");
 			return storageHubClientService.getSharedFolderMembers(folderId);
 		}catch (Exception e) {
-			logger.error("Error during get shared folder members with id: "+folderId,e);
+			LOGGER.error("Error during get shared folder members with id: "+folderId,e);
 			throw new ItemNotFoundException(e.getMessage());
 		}
 	}
@@ -445,7 +471,8 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	public boolean exists(String name, String folderId)
 		throws InternalErrorException, ItemNotFoundException,
 		WrongItemTypeException {
-
+		
+		LOGGER.debug("called exists");
 		List<WorkspaceItem> foundItems = find(name, folderId);
 
 		return foundItems!=null && foundItems.size()>0?true:false;
@@ -459,6 +486,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		throws InternalErrorException, ItemNotFoundException,
 		WrongItemTypeException {
 
+		LOGGER.debug("called find");
 		Validate.notNull(folderId,"The input folderid is null");
 		try {
 			List<? extends Item> items = storageHubClientService.findByName(name, folderId);
@@ -472,7 +500,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			return wsItems;
 		}
 		catch (Exception e) {
-			logger.error("Error during search items with name "+name+" in the parent id: "+folderId, e);
+			LOGGER.error("Error during search items with name "+name+" in the parent id: "+folderId, e);
 			throw new InternalErrorException(e.getMessage());
 		}
 	}
@@ -486,13 +514,15 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	 * @throws Exception the exception
 	 */
 	public WorkspaceItem getRootSharedFolder(String itemId) throws Exception {
+		
+		LOGGER.debug("called get Root Shared Folder");
 		Validate.notNull(itemId,"The input itemId is null");
 
 		try{
 			FolderItem folderItem = storageHubClientService.getRootSharedFolder(itemId);
 			return HLMapper.toWorkspaceItem(folderItem);
 		}catch(Exception e){
-			logger.error("Get root shared folder error: ", e);
+			LOGGER.error("Get root shared folder error: ", e);
 			throw new Exception("Error on getting the root shared folder. "+e.getMessage());
 		}
 	}
@@ -505,14 +535,15 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	@Override
 	public void deleteItem(String itemId)
 		throws Exception {
-
+		
+		LOGGER.debug("called deleteItem");
 		Validate.notNull(itemId,"The input itemId is null");
 
 		try{
 			storageHubClientService.deleteItemById(itemId);
 
 		}catch(Exception e){
-			logger.error("Delete item by id error: "+e.getMessage());
+			LOGGER.error("Delete item by id error: "+e.getMessage());
 			throw new Exception("Erro on deleting the item. "+e.getMessage());
 		}
 	}
@@ -539,14 +570,12 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	}
 
 	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.server.tohl.Workspace#uploadFile(java.lang.String, java.io.InputStream, java.lang.String, java.lang.String)
+	 * @see org.gcube.common.storagehubwrapper.server.tohl.Workspace#uploadArchive(java.lang.String, java.io.InputStream, java.lang.String)
 	 */
 	@Override
 	public	WorkspaceItem uploadArchive(
 		String folderId, InputStream is, String extractionFolderName)
-		throws
-		WorkspaceFolderNotFoundException, InternalErrorException,
-		ItemAlreadyExistException, WrongDestinationException, InsufficientPrivilegesException{
+		throws Exception{
 
 		WorkspaceItem wsItem = null;
 		try {
@@ -554,9 +583,8 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			wsItem = HLMapper.toWorkspaceItem(item);
 		}
 		catch (Exception e) {
-			logger.error("Error during uploading the archive: "+extractionFolderName+" in the folderId: "+folderId, e);
-			String error = e.getMessage()!=null?e.getMessage():"";
-			throw new InternalErrorException("Error on uploading the archive. "+error);
+			LOGGER.error("Error occurred uploading the archive: "+extractionFolderName+" in the folderId: "+folderId, e);
+			throw e;
 		}
 
 		return wsItem;
@@ -667,7 +695,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			StreamDescriptor streamDesc = storageHubClientService.downloadFile(itemId, versionName, nodeIdsToExclude);
 			return new org.gcube.common.storagehubwrapper.shared.tohl.impl.StreamDescriptor(streamDesc.getStream(), streamDesc.getFileName(), null, null);
 		} catch (Exception e) {
-			logger.error("Error on downloading the file: "+fileName+ " with id: "+itemId, e);
+			LOGGER.error("Error on downloading the file: "+fileName+ " with id: "+itemId, e);
 			String error = e.getMessage()!=null?e.getMessage():"";
 			throw new Exception("Error on downloading the file: "+fileName+". "+error);
 		}
@@ -691,7 +719,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			StreamDescriptor streamDesc = storageHubClientService.downloadFolder(folderId, nodeIdsToExclude);
 			return new org.gcube.common.storagehubwrapper.shared.tohl.impl.StreamDescriptor(streamDesc.getStream(), streamDesc.getFileName(), null, null);
 		} catch (Exception e) {
-			logger.error("Error on downloading the folder: "+folderName+ " with id: "+folderId, e);
+			LOGGER.error("Error on downloading the folder: "+folderName+ " with id: "+folderId, e);
 			String error = e.getMessage()!=null?e.getMessage():"";
 			throw new Exception("Error on downloading the folder: "+folderName+". "+error);
 		}
@@ -722,7 +750,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 					Item movedItem = storageHubClientService.moveItem(itemId, destFolderContainer);
 					toReturnItems.add(HLMapper.toWorkspaceItem(movedItem));
 				}catch(Exception e){
-					logger.error("Error on moving the item with id: "+itemId+ " in the folder id: "+destFolderContainer.get().getId(), e);
+					LOGGER.error("Error on moving the item with id: "+itemId+ " in the folder id: "+destFolderContainer.get().getId(), e);
 					throw e;
 				}
 			}
@@ -730,7 +758,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			return toReturnItems;
 
 		} catch (Exception e) {
-			logger.error("Error on moving item in the folder with id: "+folderDestinationId + e.getMessage());
+			LOGGER.error("Error on moving item in the folder with id: "+folderDestinationId + e.getMessage());
 			String error = e.getMessage()!=null?e.getMessage():"";
 			throw new Exception("Error on moving item/s. "+error);
 		}
@@ -762,14 +790,14 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 					AbstractFileItem toReturnItem = storageHubClientService.copyFileItem(itemId, destFolderContainer, null);
 					toReturnItems.add(HLMapper.toWorkspaceItem(toReturnItem));
 				}catch(Exception e){
-					logger.error("Error on copying the item with id: "+itemId+ " in the folder id: "+destFolderContainer.get().getId(), e);
+					LOGGER.error("Error on copying the item with id: "+itemId+ " in the folder id: "+destFolderContainer.get().getId(), e);
 				}
 			}
 
 			return toReturnItems;
 
 		} catch (Exception e) {
-			logger.error("Error on copying item/items in the folder with id: "+folderDestinationId, e);
+			LOGGER.error("Error on copying item/items in the folder with id: "+folderDestinationId, e);
 			String error = e.getMessage()!=null?e.getMessage():"";
 			throw new Exception("Error on copying item/s. "+error);
 		}
@@ -784,7 +812,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	public WorkspaceItem copyFile(String itemId, String folderDestinationId) throws ItemNotFoundException, WrongDestinationException, InternalErrorException, ItemAlreadyExistException, InsufficientPrivilegesException, Exception {
 
 		List<WorkspaceItem> list = copyFileItems(Arrays.asList(itemId), folderDestinationId);
-
 		return list==null||list.isEmpty()?null:list.get(0);
 
 	}
@@ -797,7 +824,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	public WorkspaceItem moveItem(String itemId, String folderDestinationId) throws ItemNotFoundException, WrongDestinationException, InsufficientPrivilegesException, ItemAlreadyExistException, InternalErrorException, Exception{
 
 		List<WorkspaceItem> list = moveItems(Arrays.asList(itemId), folderDestinationId);
-
 		return list==null||list.isEmpty()?null:list.get(0);
 
 	}
@@ -815,7 +841,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			Item item = storageHubClientService.renameItem(itemId, newName);
 			return HLMapper.toWorkspaceItem(item);
 		}catch(Exception e){
-			logger.error("Error on renaming item with id: "+itemId, e);
+			LOGGER.error("Error on renaming item with id: "+itemId, e);
 			String error = e.getMessage()!=null?e.getMessage():"Operation not allowed";
 			throw new Exception("Error on renaming. "+error);
 		}
@@ -835,12 +861,34 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		try{
 			return storageHubClientService.getPublicLinkForFile(fileItemId);
 		}catch(Exception e){
-			logger.error("Error on getting public link: "+fileItemId, e);
+			LOGGER.error("Error on getting public link: "+fileItemId, e);
 			String error = e.getMessage()!=null?e.getMessage():"Operation not allowed";
 			throw new Exception("Error on getting public link. "+error);
 		}
 	}
 
+
+	/**
+	 * Gets the public link for file.
+	 *
+	 * @param fileItemId the file item id
+	 * @param version the version
+	 * @return the public link for file
+	 * @throws Exception the exception
+	 */
+	public URL getPublicLinkForFile(String fileItemId, String version) throws Exception{
+
+		try{
+			Validate.notNull(fileItemId,"The fileItemId is null");
+			return storageHubClientService.getPublicLinkForFileVersion(fileItemId, version);
+		}catch(Exception e){
+			LOGGER.error("Error on getting public link for file: "+fileItemId +" with version: "+version, e);
+			String error = e.getMessage()!=null?e.getMessage():"Operation not allowed";
+			throw new Exception("Error on getting public link for file: "+fileItemId +" with version: "+version+". Error: "+error);
+		}
+	}
+
+
 	/**
 	 * Gets the list versions for file.
 	 *
@@ -849,11 +897,13 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	 * @throws Exception the exception
 	 */
 	public List<WorkspaceFileVersion> getListVersionsForFile(String fileItemId) throws Exception{
+		
 		try{
+			Validate.notNull(fileItemId,"The fileItemId is null");
 			List<Version> versions = storageHubClientService.getListVersions(fileItemId);
 
 			if(versions==null || versions.size()==0){
-				logger.info("No version found for fileItemId: "+fileItemId);
+				LOGGER.info("No version found for fileItemId: "+fileItemId);
 				return new ArrayList<WorkspaceFileVersion>(1);
 			}
 
@@ -866,7 +916,7 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 			return listVersions;
 
 		}catch(Exception e){
-			logger.error("Error on getting list of versions for: "+fileItemId, e);
+			LOGGER.error("Error on getting list of versions for: "+fileItemId, e);
 			String error = e.getMessage()!=null?e.getMessage():"Operation not allowed";
 			throw new Exception("Error on getting public link. "+error);
 		}
@@ -879,17 +929,16 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	public ItemStreamDescriptor getThumbnailData(String itemId) throws Exception{
 
 		try{
-
 			ImageContent imgContent =  storageHubClientService.getImageContent(itemId);
 			byte[] thumbBytes = imgContent.getThumbnailData();
     		if(thumbBytes==null || thumbBytes.length==0)
-    			throw new Exception("Thumbnail Data is not available for image with id: "+itemId);
+    			throw new Exception("Thumbnail Data for image with id: "+itemId +" is not available (null or empty) on SHUB");
 
 			return new org.gcube.common.storagehubwrapper.shared.tohl.impl.StreamDescriptor(new ByteArrayInputStream(thumbBytes), null, new Long(thumbBytes.length),imgContent.getMimeType());
 
 		}catch(Exception e){
-			logger.error("Error on getThumbnailData for: "+itemId, e);
-			throw new Exception("Error on getting the Thumbnail. "+e.getMessage());
+			LOGGER.warn("Error on getting thumbnail data for: "+itemId);
+			throw new Exception("Error on getting the thumbnail. "+e.getMessage());
 		}
 	}
 
@@ -901,84 +950,256 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	public Map<String, Object> getMetadata(String itemId) throws Exception {
 
 		try{
-
 			return storageHubClientService.getMetadata(itemId);
 
 		}catch(Exception e){
-			logger.error("Error on getting Metadata for: "+itemId, e);
+			LOGGER.error("Error on getting Metadata for: "+itemId, e);
 			throw new Exception("Error on getting Metadata for: "+itemId);
 		}
 	}
+	
+	/**
+	 * Gets the total items.
+	 *
+	 * @return the total items
+	 * @throws Exception the exception
+	 */
+	@Override
+	public long getTotalItems() throws Exception{
+
+		try{
+			return storageHubClientService.getTotalItems();
+		}catch(Exception e){
+			String error = "Error on getting total items: ";
+			LOGGER.error(error,e);
+			throw new Exception(error,e);
+		}
+	}
+	
+	
+	/**
+	 * Gets the disk usage.
+	 *
+	 * @return the disk usage
+	 * @throws Exception the exception
+	 */
+	@Override
+	public long getDiskUsage() throws Exception{
+		try{
+			return storageHubClientService.getTotalItems();
+		}catch(Exception e){
+			String error = "Error on getting disk usage: ";
+			LOGGER.error(error,e);
+			throw new Exception(error,e);
+		}
+	}
+	
+
+	/**
+	 * Gets the accounting.
+	 *
+	 * @param itemId the item id
+	 * @return the accounting
+	 * @throws Exception the exception
+	 */
+	public List<AccountEntry> getAccounting(String itemId) throws Exception{
+
+		try{
+			Item theItem = storageHubClientService.getItem(itemId, withAccounting, withMapProperties);
+			return HLMapper.toAccountingEntries(theItem);
+		}catch(Exception e){
+			LOGGER.error("Error on getAccounting for: "+itemId, e);
+			throw new Exception("Error on getAccounting for: "+itemId);
+		}
+	}
 
 
+	/**
+	 * Gets the gcube item properties.
+	 *
+	 * @param gcubeItemId the gcube item id
+	 * @return the gcube item properties
+	 * @throws Exception the exception
+	 */
+	public PropertyMap getGcubeItemProperties(String gcubeItemId) throws Exception {
+		
+		try{
+			Metadata theMetadata = storageHubClientService.getGcubeItemProperties(gcubeItemId);
+			return HLMapper.toPropertyMap(theMetadata);
+		}catch(Exception e){
+			String error = "Error on getting gcube item properties for: "+gcubeItemId;
+			LOGGER.error(error, e);
+			throw new Exception(error);
+		}
+	}
+	
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+	/* (non-Javadoc)
+	 * @see org.gcube.common.storagehubwrapper.server.tohl.Workspace#updateMetadata(java.lang.String, java.util.Map)
+	 */
+	public void updateMetadata(String itemId, Map<String,Object> mapObjs) throws Exception {
+		
+		try{
+			storageHubClientService.setMetadata(itemId,mapObjs);
+		}catch(Exception e){
+			String error = "Error on adding properties to "+GcubeItem.class.getSimpleName()+" with id: "+itemId;
+			LOGGER.error(error, e);
+			throw new Exception(error);
+		}
+	}
+	
+	
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#createExternalUrl(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
 	 */
 	@Override
-	public URLFileItem createExternalUrl(
-		String name, String description, String url, String destinationFolderId)
-		throws InsufficientPrivilegesException,
-		WorkspaceFolderNotFoundException, InternalErrorException,
-		ItemAlreadyExistException, WrongDestinationException, IOException {
-
-		// TODO Auto-generated method stub
-		return null;
+	public URLItem createURL(
+		String fileName, String fileDescription, String url, String destinationFolderId)
+		throws Exception {
+		
+		Validate.notNull(url, "The input URL is null");
+		Validate.notEmpty(url, "The input URL is empty");
+		URL theURL = null;
+		try {
+			theURL = new URL(url);
+		}catch (Exception e) {
+			String error = "The input URL "+url+" seems not to be a valid URL";
+			LOGGER.error(error, e);
+			throw new Exception(error);
+		}
+		ExternalLink extLink = storageHubClientService.addURL(destinationFolderId, theURL, fileName, fileDescription);
+		return HLMapper.toURLFile(extLink);
+		
 	}
-
+	
+	
 	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#createExternalUrl(java.lang.String, java.lang.String, java.io.InputStream, java.lang.String)
+	 * @see org.gcube.common.storagehubwrapper.server.tohl.Workspace#setFolderAsPublic(java.lang.String, boolean)
 	 */
 	@Override
-	public URLFileItem createExternalUrl(
-		String name, String description, InputStream url,
-		String destinationfolderId)
-		throws InsufficientPrivilegesException, InternalErrorException,
-		ItemAlreadyExistException, WrongDestinationException,
-		WorkspaceFolderNotFoundException, IOException {
-
-		// TODO Auto-generated method stub
-		return null;
+	public boolean setFolderAsPublic(String folderId, boolean setPublic) throws Exception{
+		
+		WorkspaceItem item =  getItem(folderId);
+		if (item.isFolder()) {
+			//TODO
+			WorkspaceFolder folder = (WorkspaceFolder) item;
+			
+			//Changing access to folder only if needed
+			if(setPublic!=folder.isPublicFolder()) {
+				LOGGER.info("Changing access to folder: "+folderId+" as "+setPublic);
+				return storageHubClientService.setFolderAsPublic(folderId, setPublic);
+			}else {
+				LOGGER.info("Access to folder: "+folderId+" is already "+setPublic+" skipping operation");
+				return setPublic;
+			}
+		}else
+			throw new Exception("The item with id '"+folderId+"' is not a folder");
 	}
+	
+	/**
+	 * Can user write into folder.
+	 *
+	 * @param folderId the folder id
+	 * @return true, if successful
+	 * @throws Exception the exception
+	 */
+	@Override
+	public boolean canUserWriteIntoFolder(String folderId) throws Exception{
+		
+		WorkspaceItem item =  getItem(folderId);
+		if (item.isFolder()) {
+			return storageHubClientService.canWrite(folderId);
+		}else
+			throw new Exception("The item with id '"+folderId+"' is not a folder");
+	}
+
+	
+	
+	/*
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * TODO
+	 * 
+	 * FOLLOWING METHODS ARE NOT IMPLEMENTED;
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 * 
+	 */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -995,40 +1216,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 	}
 
 
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getItemByPath(java.lang.String)
-	 */
-	@Override
-	public WorkspaceItem getItemByPath(String path)
-		throws ItemNotFoundException {
-
-		// TODO Auto-generated method stub
-		return null;
-	}
-
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#remove(java.lang.String, java.lang.String)
-	 */
-	@Override
-	public void remove(String itemName, String folderId)
-		throws ItemNotFoundException, InternalErrorException,
-		InsufficientPrivilegesException, WrongItemTypeException {
-
-		// TODO Auto-generated method stub
-
-	}
-
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#searchByName(java.lang.String, java.lang.String)
-	 */
-	@Override
-	public List<WorkspaceItem> searchByName(String name, String folderId)
-		throws InternalErrorException {
-
-		// TODO Auto-generated method stub
-		return null;
-	}
-
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#searchByMimeType(java.lang.String)
 	 */
@@ -1040,17 +1227,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getFolderItems(org.gcube.common.storagehub.model.types.GenericItemType)
-	 */
-	@Override
-	public List<WorkspaceItem> getFolderItems(GenericItemType type)
-		throws InternalErrorException {
-
-		// TODO Auto-generated method stub
-		return null;
-	}
-
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#createSharedFolder(java.lang.String, java.lang.String, java.util.List, java.lang.String)
 	 */
@@ -1066,21 +1242,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#createSharedFolder(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean)
-	 */
-	@Override
-	public WorkspaceSharedFolder createSharedFolder(
-		String name, String description, String groupId,
-		String destinationFolderId, String displayName, boolean isVREFolder)
-		throws InternalErrorException, InsufficientPrivilegesException,
-		ItemAlreadyExistException, WrongDestinationException,
-		ItemNotFoundException, WorkspaceFolderNotFoundException {
-
-		// TODO Auto-generated method stub
-		return null;
-	}
-
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#shareFolder(java.util.List, java.lang.String)
 	 */
@@ -1159,29 +1320,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		return null;
 	}
 
-
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getDiskUsage()
-	 */
-	@Override
-	public long getDiskUsage()
-		throws InternalErrorException {
-
-		// TODO Auto-generated method stub
-		return 0;
-	}
-
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getTotalItems()
-	 */
-	@Override
-	public int getTotalItems()
-		throws InternalErrorException {
-
-		// TODO Auto-generated method stub
-		return 0;
-	}
-
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#createVREFolder(java.lang.String, java.lang.String, java.lang.String, org.gcube.portal.storagehubwrapper.shared.ACLType)
 	 */
@@ -1196,27 +1334,6 @@ public final class WorkspaceStorageHubClientService implements Workspace{
 		return null;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getGroup(java.lang.String)
-	 */
-	@Override
-	public String getGroup(String groupId)
-		throws InternalErrorException {
-
-		// TODO Auto-generated method stub
-		return null;
-	}
-
-	/* (non-Javadoc)
-	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#isGroup(java.lang.String)
-	 */
-	@Override
-	public boolean isGroup(String groupId)
-		throws InternalErrorException {
-
-		// TODO Auto-generated method stub
-		return false;
-	}
 
 	/* (non-Javadoc)
 	 * @see org.gcube.portal.storagehubwrapper.shared.Workspace#getPublicFolders()
